#/usr/bin/perl
# Auteur : Arthur Provenier

use utf8;
use HTML::Entities;

binmode STDOUT, ":utf8";

system("rm resultat/fic_eti.txt resultat/fic_eti1.txt resultat/temporaire.txt");

=for comment
Script projet BAO 2017
Auteurs initiaux	-	Serge Fleury, Jean-Michel Daube
Modification		-	Arthur Provenier
Usage				-	perl projet_perl.pl arborescence_a_parcourir (rubrique)
Données				-	une arborescence à parcourir, une rubrique (optionnel)
Résultat			-	un fichier rubrique.txt et un fichier rubrique.xml

Chaque fichier contient les titres et les descriptions extraits des flux RSS
Le fichier xml produit est de la forme
<PARCOURS>
	<FILTRAGE>
		<ITEM>
			<TITRE></TITRE>
			<DESCRIPTION></DESCRIPTION>
		</ITEM>
	</FILTRAGE>
</PARCOURS>

Utilisation de tokenise-utf8.pl pour ségmenter et TreeTager pour annoter le POS
=cut
#-------------------------------------------------------------------------------------------
# Début du programme principale

my $rep = "$ARGV[0]";		# Contient le chemin de l'arborescence
my $rubrique = "$ARGV[1]";	# Contient le nom de la rubrique choisie
my %memoire = ();	# Utiliser pour éviter les doublons en sortie

$rep =~ s/[\/]$//;	# Pour que le répertoire ne finisse pas par '/'

# Ouverture des fichiers
open my $out, ">:encoding(UTF-8)", "resultat/rubrique-$rubrique.txt";
open my $out1, ">>:encoding(UTF-8)", "resultat/rubrique-$rubrique.xml";

# Écriture des balises xml du fichier xml
print $out1 "<?xml version=\"1.0\" encoding=\"utf-8\" ?>\n\n";
print $out1 "<PARCOURS>\n";
print $out1 "<NOM>Arthur Provenier</NOM>\n";
print $out1 "<FILTRAGE>\n\n";

&parcoursArborescenceFichiers($rep);	# On lance la récursion sur le dossier à parcourir

# Fermeture des balises du fichier xml
print $out1 "\n</FILTRAGE>\n";
print $out1 "\n</PARCOURS>\n";

close $out;
close $out1;

exit;
# Fin du programme principale
#-------------------------------------------------------------------------------------------
# Parcourir l'arborescence de façon récursive
sub parcoursArborescenceFichiers
{
	my $path = shift;

	opendir(DIR, $path) or die "Impossible d'ouvrir $path : $!\n";
	my @files = readdir(DIR);
	closedir(DIR);

	foreach my $file (@files)
	{
		next if $file =~ /(^\.\.?$)|(^\.DS_Store$)/;

		$file = $path.'/'.$file;

		if (-d $file)
		{
			#print "nouveau répertoire ->\t$file\n";
			&parcoursArborescenceFichiers($file);	# on recommence un tour ...
			#print "fin du répertoire ->\t$file\n";
		}
		elsif (-f $file)
		{
			if ($file =~ /$rubrique.+\.xml$/)
			{
				open my $input, "<:encoding(utf-8)", $file;

				my $texte = "";
				while (my $ligne = <$input>)
				{
					chomp $ligne;
					$ligne =~ s/\r//g;
					$texte .= $ligne;
				} # Fin while lecture
				$texte =~ s/> +</></g;
				$texte = decode_entities($texte); # suppression des entités html

				while ($texte =~ /<item>(?:<link>[^<]+<\/link>)?<title>(.+?)<\/title>.+?<description>(.+?)<\/description>/g)
				{
					my $titre = $1;
					my $description = $2;

					if (!(exists $memoire{$titre}))
					{
						$description =~ s/<br clear='all'.+?$//g; # Suppression des pubs le motif extraits

						my ($titrexml, $descriptionxml) = &etiquettage($titre, $description);

						binmode STDOUT, ":utf8";
						print $out "$titre.\n$description\n\n"; # Impression dans le fichier texte
						print $out1 "<item>\n<titre>$titrexml</titre>\n<description>$descriptionxml</description>\n</item>\n\n";
						# Impression dans le fichier xml

						$memoire{$titre} = 1; # éviter les doublons en stockant dans un %hash avec clé unique
					} # Fin if éviter les doublons et écriture dans fichiers
				} # Fin while extraction motif
				close $input;
			} # Fin if rubrique
		} # Fin elsif
	} # Fin foreach
} # Fin sub

#-------------------------------------------------------------------------------------------
# Étiquettage via Treetagger
sub etiquettage
{
	my ($t, $d) = @_;

	open my $temp, ">:encoding(utf-8)", "resultat/temporaire.txt";
	print $temp "$t";
	system("perl -f tokenise-utf8.pl resultat/temporaire.txt | tree-tagger -token -lemma /Users/arthurprovenier/TreeTagger/lib/french-utf8.par -no-unknown > resultat/fic_eti.txt");
	system("perl treetagger2xml-utf8.pl resultat/fic_eti.txt utf8");

	open my $fic, "<:encoding(utf-8)", "resultat/fic_eti.txt.xml";
	my $t_xml = "";
	my $trash = <$fic>;	# mettre la première ligne à la poubelle
	while (my $ligne = <$fic>)
	{
		$t_xml .= $ligne;
	}
	close $fic;
	close $temp;

	###########

	open my $temp, ">:encoding(utf-8)", "resultat/temporaire.txt";
	print $temp "$d";
	system("perl -f tokenise-utf8.pl resultat/temporaire.txt | tree-tagger -token -lemma /Users/arthurprovenier/TreeTagger/lib/french-utf8.par -no-unknown > resultat/fic_eti1.txt");
	system("perl treetagger2xml-utf8.pl resultat/fic_eti1.txt utf8");

	open my $fic, "<:encoding(utf-8)", "resultat/fic_eti1.txt.xml";
	my $d_xml = "";
	my $trash = <$fic>;	# mettre la première ligne à la poubelle
	while (my $ligne = <$fic>)
	{
		$d_xml .= $ligne;
	}
	close $fic;
	close $temp;

	return ($t_xml, $d_xml);
}
